@@ -11,12 +11,13 @@ use eva_common::services;
 use eva_common::SLEEP_STEP;
 use lazy_static::lazy_static;
 use log::{Level, Log};
+use serde::Deserialize;
 use std::collections::HashMap;
 use std::io::Read;
 use std::sync::atomic;
 use std::sync::Arc;
 use std::sync::Mutex;
-use std::time::Duration;
+use std::time::{Duration, Instant};
 use tokio::io::AsyncReadExt;
 use tokio::signal::unix::{signal, SignalKind};
 use tokio::time::sleep;
@@ -103,6 +104,37 @@ pub async fn safe_rpc_call(
         .map_err(Into::into)
 }
 
+#[inline]
+pub async fn wait_core(rpc: &RpcClient, timeout: Duration, wait_forever: bool) -> EResult<()> {
+    #[derive(Deserialize)]
+    struct TR {
+        active: bool,
+    }
+    let wait_until = Instant::now() + timeout;
+    loop {
+        if let Ok(ev) = safe_rpc_call(
+            rpc,
+            "eva.core",
+            "test",
+            elbus::empty_payload!(),
+            QoS::Processed,
+            timeout,
+        )
+        .await
+        {
+            if let Ok(result) = rmp_serde::from_slice::<TR>(ev.payload()) {
+                if result.active {
+                    return Ok(());
+                }
+            }
+        }
+        tokio::time::sleep(SLEEP_STEP).await;
+        if !wait_forever && wait_until >= Instant::now() {
+            return Err(Error::timeout());
+        }
+    }
+}
+
 static ACTIVE: atomic::AtomicBool = atomic::AtomicBool::new(false);
 
 #[inline]
